Object subclass: #ClientManager
	instanceVariableNames: 'clientsToConnect clients objectsVisibleByClients '
	classVariableNames: 'Current '
	poolDictionaries: ''
	category: 'Scrabble-Server'!

!ClientManager methodsFor: 'server processes' stamp: 'JMV 7/20/1999 23:19'!
process
	"Scrabble Server main loop"

	| continue |
	continue _ true.
	[ continue ] whileTrue: [
		continue _ self connectNewClients.
		self removeLostClients.
		self processIncomingMessages.
		VirtualPlayroom current processOneCycle.
		(Delay forMilliseconds: 1) wait.
	].
! !

!ClientManager methodsFor: 'server processes' stamp: 'JMV 7/13/1999 14:04'!
start
	"Start Scrabble Server"

	VirtualPlayroom current. "Inicialize VirtualPlayroom if necessary"
	clients _ OrderedCollection new.
	Transcript cr.
	Transcript show: 'Initializating Scrabble Server'; cr.
	Transcript show: 'starting network'; cr.
	Socket initializeNetwork.
	Transcript show: 'This server IP address is: ', 
		NetNameResolver localAddressString; cr.
	Transcript show: 'Server active'; cr.
	clientsToConnect _ SharedQueue new.

	[ self waitForConnection ] forkAt: Processor userInterruptPriority.
	self process.

	self deinitialize! !

!ClientManager methodsFor: 'server processes' stamp: 'JMV 7/9/1999 17:18'!
waitForConnection
	| initialSocket messageString clientIPAddress clientPort definitiveSocket |

	initialSocket _ SimpleClientSocket new.
	initialSocket listenOn: 54323.
	[ Sensor anyButtonPressed ] whileFalse: [
		initialSocket waitForConnectionUntil: (Socket deadlineSecs: 1).
		initialSocket isConnected ifTrue: [
			[ initialSocket dataAvailable ] whileFalse: [
				Transcript show: 'Client connecting. Awaiting for data.'; cr.
				(Delay forMilliseconds: 100) wait].
			messageString _ initialSocket getResponse.
			clientIPAddress _ NetNameResolver addressFromString: messageString substrings first.
			clientPort _ messageString substrings last asNumber. 
			definitiveSocket _ SimpleClientSocket new.
			definitiveSocket connectTo: clientIPAddress port: clientPort.
			clientsToConnect nextPut: definitiveSocket.
			"initialSocket disconnect."
			initialSocket closeAndDestroy: 3.
			initialSocket _ SimpleClientSocket new.
			initialSocket listenOn: 54323.
		].
	].
	initialSocket closeAndDestroy.
	clientsToConnect nextPut: #stopServer.! !


!ClientManager methodsFor: 'services' stamp: 'JMV 7/13/1999 22:21'!
convertToObject: anObjectIDOrString

	| object |
	(anObjectIDOrString beginsWith: 'objectID:') ifTrue: [
		object _ self objectsVisibleByClients 
			at: anObjectIDOrString 
			ifAbsent: [nil] ]
	ifFalse: [
		object _ anObjectIDOrString ].
	^ object! !

!ClientManager methodsFor: 'services' stamp: 'JMV 7/14/1999 23:52'!
convertToObjectID: anObjectWithIDOrString

	| id |
	(anObjectWithIDOrString isKindOf: ObjectWithID) ifTrue: [
		id _ anObjectWithIDOrString objectID ]
	ifFalse: [
		id _ anObjectWithIDOrString asString ].
	^ id.! !

!ClientManager methodsFor: 'services' stamp: 'JMV 7/20/1999 23:09'!
notifyInterestedParties: anObjectID event: aSymbol withArguments: anArray
	| aLiteralOrIDArray messageString |

	aLiteralOrIDArray _ Array new: anArray size.
	1 to: aLiteralOrIDArray size do: [ :i |
		aLiteralOrIDArray at: i put: (self convertToObjectID: (anArray at: i)) ].
	messageString _ 'evt', self separatorCharacter asString, 
		anObjectID, self separatorCharacter asString, aSymbol asString.
	aLiteralOrIDArray do: [ :each |
		messageString _ messageString, self separatorCharacter asString, each asString ].

	clients do: [ :eachSocket |
		eachSocket isConnected ifTrue: [
			eachSocket sendCommand: messageString ] ].
! !


!ClientManager methodsFor: 'housekeeping' stamp: 'JMV 7/9/1999 12:42'!
connectNewClients

	| newSocket |

	[ clientsToConnect size > 0 ] whileTrue: [
		newSocket _ clientsToConnect next.
		newSocket = #stopServer ifTrue: [
			^ false ]
		ifFalse: [
			newSocket waitForConnectionUntil: (Socket deadlineSecs: 1).
			(newSocket isConnected) ifFalse: [
				self error: 'Could not entablish requested connection' ].
			Transcript show: 'Connection entablished: ', 
				newSocket remoteAddress printString; cr.
			clients add: newSocket.
		].
	].
	^ true! !

!ClientManager methodsFor: 'housekeeping' stamp: 'JMV 7/20/1999 23:11'!
deinitialize
	"Close the Server"

	clientsToConnect _ nil.
	clients do: [ :eachSocket |
		eachSocket closeAndDestroy: 3 ].
	clients _ nil.
	"outgoingMessages _ nil."
	"objectsVisibleByClients _ nil."
	Transcript show: 'Server disconnected'; cr.! !

!ClientManager methodsFor: 'housekeeping' stamp: 'JMV 3/26/2000 10:44'!
removeLostClients
	"Get rid of clients whose connection was lost "

	| socketsToRemove |

	socketsToRemove _ clients select: [ :each | each isConnected not ].
	socketsToRemove do: [ :each |
		Transcript show: 
			'Lost connection with: ', each remoteAddress printString; cr.
		each closeAndDestroy: 3 .
	].
	clients removeAll: socketsToRemove.
! !


!ClientManager methodsFor: 'servidor' stamp: 'JMV 7/22/1999 22:27'!
processIncomingMessage: messageString fromClient: aSocket

	| objectID receiver message parts arguments result answer |
"	Transcript show: messageString; cr."
	parts _ messageString substrings: self separatorCharacter.
	objectID _ parts first.
	message _ (parts at: 2) asSymbol.
	arguments _ Array new: parts size - 2.
	1 to: arguments size do: [ :i |
		arguments at: i put: (self convertToObject: (parts at: i+2)) ].
	objectID = 'VirtualPlayroom' ifTrue: [
		receiver _ VirtualPlayroom current ]
	ifFalse: [
		(self objectsVisibleByClients at: objectID ifAbsent: [nil]) isNil ifTrue: [
			self error: 'Invalid ObjectID received' ].
		receiver _ self objectsVisibleByClients at: objectID ].
	result _ receiver perform: message withArguments: arguments.
"	Transcript show: result asString; cr."

	(result class = OrderedCollection) ifTrue: [
		answer _ 'ocl'.
		result do: [ :each |
			answer _ answer, self separatorCharacter asString, (self convertToObjectID: each) ] ]
	ifFalse: [
		answer _ 'rsp', self separatorCharacter asString, (self convertToObjectID: result) ].
	aSocket sendCommand: answer.! !

!ClientManager methodsFor: 'servidor' stamp: 'JMV 7/22/1999 22:27'!
processIncomingMessages

	| messageString |

	clients do: [ :eachSocket |
		eachSocket isConnected ifTrue: [
			eachSocket dataAvailable ifTrue: [
				messageString _ eachSocket getData. "JMV data o response?"
				messageString size > 0 ifTrue: [
"					Transcript show: messageString; cr."
					self processIncomingMessage: 
							(messageString truncateTo: messageString size-2)
						fromClient: eachSocket. ] ] ] ]! !


!ClientManager methodsFor: 'accesing' stamp: 'JMV 7/13/1999 20:47'!
addVisibleByClients: anObjectWithID

	self objectsVisibleByClients at: anObjectWithID objectID put: anObjectWithID.! !

!ClientManager methodsFor: 'accesing' stamp: 'JMV 7/16/1999 18:00'!
objectsVisibleByClients
	"Dictionary of objects visible by clients via their objectID"

	objectsVisibleByClients isNil ifTrue: [
		objectsVisibleByClients _ "WeakValue"Dictionary new.
	].
	^ objectsVisibleByClients! !

!ClientManager methodsFor: 'accesing' stamp: 'JMV 7/24/1999 11:51'!
removeVisibleByClients: anObjectWithID

	self objectsVisibleByClients removeKey: anObjectWithID objectID! !

!ClientManager methodsFor: 'accesing' stamp: 'JMV 3/26/2000 09:14'!
separatorCharacter
	"The character we use as a separator in TCP/IP messages "
	^ $\! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ClientManager class
	instanceVariableNames: ''!

!ClientManager class methodsFor: 'instance creation' stamp: 'JMV 7/7/1999 19:44'!
current

	Current isNil ifTrue: [
		Current _ self new ].
	^ Current! !
